from .containers import Container
from .context import InjectionContext, SyncInjectionContext
from .decorators import inject
from .markers import Inject
from .providers import Callable, Object, Provider, Singleton

__all__ = [
    "Container",
    "InjectionContext",
    "SyncInjectionContext",
    "inject",
    "Inject",
    "Callable",
    "Provider",
    "Singleton",
    "Object",
]
__version__ = "0.10.0"



import dataclasses
from typing import Annotated

import pytest

from aioinject import Callable, Container, Inject


class _A:
    pass


@dataclasses.dataclass
class _B:
    a: Annotated[_A, Inject]


@dataclasses.dataclass
class _C:
    b: Annotated[_B, Inject]


@pytest.fixture
def container() -> Container:
    container = Container()
    container.register(Callable(_A))
    container.register(Callable(_B))
    container.register(Callable(_C))
    return container



import pytest

from aioinject import Singleton


class _Test:
    pass


def test_identity() -> None:
    provider = Singleton(_Test)
    instance = provider.provide_sync()
    assert instance is provider.provide_sync()


@pytest.mark.anyio
async def test_identity_async() -> None:
    provider = Singleton(_Test)
    instance = provider.provide_sync()
    assert instance is provider.provide_sync() is await provider.provide()


@pytest.mark.anyio
async def test_async_function() -> None:
    async def create_test() -> _Test:
        return _Test()

    provider = Singleton(factory=create_test)
    instance = await provider.provide()
    assert instance is await provider.provide()



import contextlib
import inspect
import typing
from collections.abc import Awaitable, Callable, Iterator
from contextlib import (
    AbstractAsyncContextManager,
    AbstractContextManager,
    AsyncExitStack,
)
from typing import Any, ParamSpec, TypeVar, cast

from aioinject.markers import Inject

_T = TypeVar("_T")
_P = ParamSpec("_P")

_sentinel = object()


def clear_wrapper(wrapper: Callable[_P, _T]) -> Callable[_P, _T]:
    inject_annotations = get_inject_annotations(wrapper)
    signature = inspect.signature(wrapper)
    new_params = tuple(
        p
        for p in signature.parameters.values()
        if p.name not in inject_annotations
    )
    wrapper.__signature__ = signature.replace(  # type: ignore[attr-defined]
        parameters=new_params,
    )
    for name in inject_annotations:
        del wrapper.__annotations__[name]
    return wrapper


def get_inject_annotations(function: Callable[..., Any]) -> dict[str, Any]:
    with remove_annotation(function.__annotations__, "return"):
        return {
            name: annotation
            for name, annotation in typing.get_type_hints(
                function,
                include_extras=True,
            ).items()
            if any(
                isinstance(arg, Inject) or arg is Inject
                for arg in typing.get_args(annotation)
            )
        }


def enter_context_maybe(
    resolved: AbstractContextManager[_T]
    | AbstractAsyncContextManager[_T]
    | _T,
    stack: AsyncExitStack,
) -> _T | Awaitable[_T]:
    if isinstance(resolved, contextlib.ContextDecorator):
        return stack.enter_context(resolved)  # type: ignore[arg-type]

    if isinstance(resolved, contextlib.AsyncContextDecorator):
        return stack.enter_async_context(
            resolved,  # type: ignore[arg-type]
        )
    return resolved  # type: ignore[return-value]


async def await_maybe(value: _T | Awaitable[_T]) -> _T:
    if inspect.isawaitable(value):
        return await value
    return cast(_T, value)


@contextlib.contextmanager
def remove_annotation(
    annotations: dict[str, Any],
    name: str,
) -> Iterator[None]:
    annotation = annotations.pop(name, _sentinel)
    yield
    if annotation is not _sentinel:
        annotations[name] = annotation
